=head1 NAME

PApp::CGI -  use PApp in a CGI environment

=head1 SYNOPSIS

=head1 DESCRIPTION

Don't use this module (YET): a) it isn't finished and b) you need to use
CGI::SpeedyCGI to get any reasonable speed.

=over 4

=cut

package PApp::CGI;

require Exporter;

@ISA = qw(Exporter);
$VERSION = 0.04;
@EXPORT = qw();
$VERSION = 0.04;

1;

=back

=head1 AUTHOR

Marc Lehmann <pcg@goof.com>

=cut

